@@ -4,8 +4,10 @@
 #
 # This code is covered under the terms of the GNU Library General
 # Public License as described in the bup LICENSE file.
+
+from io import BytesIO
 import errno, os, sys, stat, time, pwd, grp, socket, struct
-from cStringIO import StringIO
+
 from bup import vint, xstat
 from bup.drecurse import recursive_dirlist
 from bup.helpers import add_error, mkdirp, log, is_superuser, format_filesize
@@ -212,6 +214,8 @@ class Metadata:
     # must be non-negative and < 10**9.
 
     def _add_common(self, path, st):
+        assert(st.st_uid >= 0)
+        assert(st.st_gid >= 0)
         self.uid = st.st_uid
         self.gid = st.st_gid
         self.atime = st.st_atime
@@ -304,14 +308,14 @@ class Metadata:
         st = None
         try:
             st = xstat.lstat(path)
-        except OSError, e:
+        except OSError as e:
             if e.errno != errno.ENOENT:
                 raise
         if st:
             if stat.S_ISDIR(st.st_mode):
                 try:
                     os.rmdir(path)
-                except OSError, e:
+                except OSError as e:
                     if e.errno in (errno.ENOTEMPTY, errno.EEXIST):
                         msg = 'refusing to overwrite non-empty dir ' + path
                         raise Exception(msg)
@@ -321,24 +325,24 @@ class Metadata:
 
         if stat.S_ISREG(self.mode):
             assert(self._recognized_file_type())
-            fd = os.open(path, os.O_CREAT|os.O_WRONLY|os.O_EXCL, 0600)
+            fd = os.open(path, os.O_CREAT|os.O_WRONLY|os.O_EXCL, 0o600)
             os.close(fd)
         elif stat.S_ISDIR(self.mode):
             assert(self._recognized_file_type())
-            os.mkdir(path, 0700)
+            os.mkdir(path, 0o700)
         elif stat.S_ISCHR(self.mode):
             assert(self._recognized_file_type())
-            os.mknod(path, 0600 | stat.S_IFCHR, self.rdev)
+            os.mknod(path, 0o600 | stat.S_IFCHR, self.rdev)
         elif stat.S_ISBLK(self.mode):
             assert(self._recognized_file_type())
-            os.mknod(path, 0600 | stat.S_IFBLK, self.rdev)
+            os.mknod(path, 0o600 | stat.S_IFBLK, self.rdev)
         elif stat.S_ISFIFO(self.mode):
             assert(self._recognized_file_type())
-            os.mknod(path, 0600 | stat.S_IFIFO)
+            os.mknod(path, 0o600 | stat.S_IFIFO)
         elif stat.S_ISSOCK(self.mode):
             try:
-                os.mknod(path, 0600 | stat.S_IFSOCK)
-            except OSError, e:
+                os.mknod(path, 0o600 | stat.S_IFSOCK)
+            except OSError as e:
                 if e.errno in (errno.EINVAL, errno.EPERM):
                     s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
                     s.bind(path)
@@ -350,7 +354,7 @@ class Metadata:
                 # on MacOS, symlink() permissions depend on umask, and there's
                 # no way to chown a symlink after creating it, so we have to
                 # be careful here!
-                oldumask = os.umask((self.mode & 0777) ^ 0777)
+                oldumask = os.umask((self.mode & 0o777) ^ 0o777)
                 try:
                     os.symlink(self.symlink_target, path)
                 finally:
@@ -370,7 +374,7 @@ class Metadata:
         if lutime and stat.S_ISLNK(self.mode):
             try:
                 lutime(path, (self.atime, self.mtime))
-            except OSError, e:
+            except OSError as e:
                 if e.errno == errno.EACCES:
                     raise ApplyError('lutime: %s' % e)
                 else:
@@ -378,7 +382,7 @@ class Metadata:
         else:
             try:
                 utime(path, (self.atime, self.mtime))
-            except OSError, e:
+            except OSError as e:
                 if e.errno == errno.EACCES:
                     raise ApplyError('utime: %s' % e)
                 else:
@@ -412,7 +416,7 @@ class Metadata:
         if uid != -1 or gid != -1:
             try:
                 os.lchown(path, uid, gid)
-            except OSError, e:
+            except OSError as e:
                 if e.errno == errno.EPERM:
                     add_error('lchown: %s' %  e)
                 elif sys.platform.startswith('cygwin') \
@@ -449,7 +453,7 @@ class Metadata:
         try:
             if stat.S_ISLNK(st.st_mode):
                 self.symlink_target = os.readlink(path)
-        except OSError, e:
+        except OSError as e:
             add_error('readlink: %s' % e)
 
     def _encode_symlink_target(self):
@@ -484,7 +488,8 @@ class Metadata:
     # The numeric/text distinction only matters when reading/restoring
     # a stored record.
     def _add_posix1e_acl(self, path, st):
-        if not posix1e: return
+        if not posix1e or not posix1e.HAS_EXTENDED_CHECK:
+            return
         if not stat.S_ISLNK(st.st_mode):
             acls = None
             def_acls = None
@@ -495,7 +500,7 @@ class Metadata:
                     if stat.S_ISDIR(st.st_mode):
                         def_acl = posix1e.ACL(filedef=path)
                         def_acls = [def_acl, def_acl]
-            except EnvironmentError, e:
+            except EnvironmentError as e:
                 if e.errno not in (errno.EOPNOTSUPP, errno.ENOSYS):
                     raise
             if acls:
@@ -532,7 +537,7 @@ class Metadata:
         def apply_acl(acl_rep, kind):
             try:
                 acl = posix1e.ACL(text = acl_rep)
-            except IOError, e:
+            except IOError as e:
                 if e.errno == 0:
                     # pylibacl appears to return an IOError with errno
                     # set to 0 if a group referred to by the ACL rep
@@ -543,7 +548,7 @@ class Metadata:
                     raise
             try:
                 acl.applyto(path, kind)
-            except IOError, e:
+            except IOError as e:
                 if e.errno == errno.EPERM or e.errno == errno.EOPNOTSUPP:
                     raise ApplyError('POSIX1e ACL applyto: %s' % e)
                 else:
@@ -577,7 +582,7 @@ class Metadata:
                 attr = get_linux_file_attr(path)
                 if attr != 0:
                     self.linux_attr = attr
-            except OSError, e:
+            except OSError as e:
                 if e.errno == errno.EACCES:
                     add_error('read Linux attr: %s' % e)
                 elif e.errno in (errno.ENOTTY, errno.ENOSYS, errno.EOPNOTSUPP):
@@ -609,7 +614,7 @@ class Metadata:
                 return
             try:
                 set_linux_file_attr(path, self.linux_attr)
-            except OSError, e:
+            except OSError as e:
                 if e.errno in (errno.ENOTTY, errno.EOPNOTSUPP, errno.ENOSYS,
                                errno.EACCES):
                     raise ApplyError('Linux chattr: %s (0x%s)'
@@ -624,7 +629,7 @@ class Metadata:
         if not xattr: return
         try:
             self.linux_xattr = xattr.get_all(path, nofollow=True)
-        except EnvironmentError, e:
+        except EnvironmentError as e:
             if e.errno != errno.EOPNOTSUPP:
                 raise
 
@@ -643,7 +648,7 @@ class Metadata:
 
     def _load_linux_xattr_rec(self, file):
         data = vint.read_bvec(file)
-        memfile = StringIO(data)
+        memfile = BytesIO(data)
         result = []
         for i in range(vint.read_vuint(memfile)):
             key = vint.read_bvec(memfile)
@@ -661,7 +666,7 @@ class Metadata:
             return
         try:
             existing_xattrs = set(xattr.list(path, nofollow=True))
-        except IOError, e:
+        except IOError as e:
             if e.errno == errno.EACCES:
                 raise ApplyError('xattr.set %r: %s' % (path, e))
             else:
@@ -671,7 +676,7 @@ class Metadata:
                     or v != xattr.get(path, k, nofollow=True):
                 try:
                     xattr.set(path, k, v, nofollow=True)
-                except IOError, e:
+                except IOError as e:
                     if e.errno == errno.EPERM \
                             or e.errno == errno.EOPNOTSUPP:
                         raise ApplyError('xattr.set %r: %s' % (path, e))
@@ -681,8 +686,8 @@ class Metadata:
         for k in existing_xattrs:
             try:
                 xattr.remove(path, k, nofollow=True)
-            except IOError, e:
-                if e.errno == errno.EPERM:
+            except IOError as e:
+                if e.errno in (errno.EPERM, errno.EACCES):
                     raise ApplyError('xattr.remove %r: %s' % (path, e))
                 else:
                     raise
@@ -743,7 +748,7 @@ class Metadata:
         vint.write_vuint(port, _rec_tag_end)
 
     def encode(self, include_path=True):
-        port = StringIO()
+        port = BytesIO()
         self.write(port, include_path)
         return port.getvalue()
 
@@ -807,7 +812,7 @@ class Metadata:
                                self._apply_linux_xattr_rec):
             try:
                 apply_metadata(path, restore_numeric_ids=num_ids)
-            except ApplyError, e:
+            except ApplyError as e:
                 add_error(e)
 
     def same_file(self, other):
